<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Service\Sql;

use Dedi\SyliusMigrationPlugin\Definition\TableMappingDefinition;
use Dedi\SyliusMigrationPlugin\Dto\OrderMappingDefinition;
use LogicException;

class Reader extends AbstractSqlProcessor implements ReaderInterface
{
    public function read(TableMappingDefinition $tableMappingDefinition): array
    {
        if ('' !== $tableMappingDefinition->getSql()) {
            $this->logger->debug($tableMappingDefinition, $tableMappingDefinition->getSql());

            return $this->connection->executeQuery($tableMappingDefinition->getSql())->fetchAllAssociative();
        }

        $sourceQueryBuilder = $this->connection->createQueryBuilder();
        $sourceQueryBuilder
            ->from($tableMappingDefinition->getSourceTableName())
            ->addSelect($tableMappingDefinition->getSourcePrimaryKey());

        foreach ($tableMappingDefinition->getColumnMappingDefinitions() as $columnMappingDefinition) {
            if (null !== $columnMappingDefinition->getSourceColumnName()) {
                $sourceQueryBuilder
                    ->addSelect($columnMappingDefinition->getSourceColumnName());
            }

            if (null !== $columnMappingDefinition->getExpr()) {
                if (null === $columnMappingDefinition->getAlias()) {
                    throw new LogicException('Expr configuration needs an Alias');
                }
                $sourceQueryBuilder
                    ->addSelect(sprintf('%s AS %s', $columnMappingDefinition->getExpr(), $columnMappingDefinition->getAlias()));
            }
        }

        if ($tableMappingDefinition->hasFilterMapping()) {
            foreach ($tableMappingDefinition->getFilterMapping() as $filterMappingDefinition) {
                $pattern = $this->resolveSearchType($filterMappingDefinition->getType());

                $sourceQueryBuilder
                    ->andWhere(sprintf($pattern, $filterMappingDefinition->getFilterColumn(), $filterMappingDefinition->getFilterColumn()))
                    ->setParameter($filterMappingDefinition->getFilterColumn(), $filterMappingDefinition->getValue());
            }
        }

        $orderMappingDefinition = $tableMappingDefinition->getOrderMappingDefinition();

        if ($orderMappingDefinition instanceof OrderMappingDefinition) {
            $sourceQueryBuilder->orderBy($orderMappingDefinition->getColumn(), $orderMappingDefinition->getSort());
        } else {
            $sourceQueryBuilder->orderBy($tableMappingDefinition->getSourcePrimaryKey(), 'DESC');
        }

        $sql = $sourceQueryBuilder->getSQL();
        $this->logger->debug($tableMappingDefinition, $sql);

        return $this->connection->executeQuery($sql, $sourceQueryBuilder->getParameters())->fetchAllAssociative();
    }

    private function resolveSearchType(string $type)
    {
        switch ($type) {
            case 'comparison':
                return '%s = :%s';
            case 'not_comparison':
                return '%s != :%s';
            case 'like':
                return '%s LIKE :%s';
            case 'is_null':
                return '%s IS NULL';
            default:
                throw new LogicException('Unsupported type %s', $type);
        }
    }
}
